! Sensor type -3 test v2.0 -humpty Jun 2023
! Modified from original f14_compass.bas

! This example program demonstrates the
! use of the Pseudo "Orientation" sensor
! to make a simple Compass.

! Your device must have both Acceleraion 
! and Magnetic Field hardware sensors.
!----------------------------------------
! Set your local magnetic declination
! from true North

declination = 0          % +ve for East -ve for West

! Open the graphics screen with a
! White background

GR.OPEN 255,255,255,255, 0,1	% A,R,G,B,barOFF,landscape
PAUSE 1000
GR.SCREEN w, h

cx = w/2
cy = h/2
th = h/24                   % text height

if w<h then r=w/2 else r=h/2
r *= 0.7                    % radius

! Initialize the text parameters

GR.COLOR 255,0,0,0,1      % text color
GR.TEXT.SIZE th*0.7
GR.TEXT.ALIGN 2

! Draw the parameter value displays

z = TORADIANS(45)
x = COS(z) * r + cx
y = SIN(z) * r +cy
GR.TEXT.DRAW ta, x, y,"A000"    % Azimuth
z = TORADIANS(135)
x = COS(z) * r + cx
y = SIN(z) * r +cy
GR.TEXT.DRAW tp, x, y,"P000"    % Pitch
z = TORADIANS(225)
x = COS(z) * r + cx
y = SIN(z) * r +cy
GR.TEXT.DRAW tr, x, y, "R000"   % Roll

! Draw the compass labels

GR.COLOR 255,0,0,255,1      % labels color
GR.TEXT.SIZE th
GR.TEXT.DRAW k, cx, cy - r - th, "N"
GR.TEXT.DRAW k, cx + r + th, cy, "W"
GR.TEXT.DRAW k, cx - r - th ,cy, "E"
GR.TEXT.DRAW k, cx, cy + r  + th, "S"
GR.RENDER   % Show what we have drawn so far

! Now pre-draw 360 degree lines. This will be
! hidden and then shown one at a time based
! upon the Azimuth values returned

GR.COLOR 255,255,0,0,1      % line color
GR.SET.STROKE 2

DIM a[360]
FOR i = 1 TO 360
 GR.ROTATE.START i, cx, cy
 GR.LINE a[i], cx, cy, cx, cy-r	% N is straight up
 GR.ROTATE.END
 GR.HIDE a[i]
NEXT i

! Use the special Orientation Sensor -3
! It uses types 1 (acceleration) and 2 (magnetic)
! and they must be open
SENSORS.OPEN 1,2

! In an infinite loop,  read the sensor
! and show the compass line for
! the (adjusted) Azimuth

DO
 SENSORS.READ -3, az, pitch, roll
 az += declination  % Add in the declination
 az = ROUND(az)  % Round to the nearest whole number

 if az>360 then az-=360
 if az<0   then az+=360
				% az is Y clockwise from N
 N = 360-az		% therefore N is anti-clockwise from Y

 IF N = 0 THEN N = 360	% no line for 0

 GR.SHOW a[N] % Show the Azmuth Line - it points to N

 % Display the sensor's parameters

 az$ = FORMAT$("A%%%", az)
 pitch$ = FORMAT$("P%%%", pitch)
 roll$ = FORMAT$("R%%%", roll)
 GR.MODIFY ta, "text", az$
 GR.MODIFY tp, "text", pitch$
 GR.MODIFY tr, "text", roll$

 % Show the changes made, pause
 % and then hide the Azimuth line

 GR.RENDER
 PAUSE 100
 GR.HIDE a[N]

UNTIL 0  % Loop forever

END

